"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Quantize = require("quantize");
var RgbColor = require("../color/rgb");
var RgbaColor = require("../color/rgba");
exports.COLOR_INDEX_TRANSPARENT = 0xf;
function isOpaque(alpha) {
    return alpha > 0;
}
exports.isOpaque = isOpaque;
function extractOpaqueColors(rgbaHexes) {
    var opaqueHexes = rgbaHexes.filter(function (rgbaHex) {
        var comps = RgbaColor.componentsFromHex(rgbaHex);
        return isOpaque(comps[3]);
    });
    return opaqueHexes.map(function (rgbaHex) {
        var comps = RgbaColor.componentsFromHex(rgbaHex);
        return RgbColor.hexFromComponents([
            comps[0],
            comps[1],
            comps[2],
        ]);
    });
}
function getNearestColorIndex(colMap, palette, rgbComps) {
    var comps = colMap.map(rgbComps);
    var minDist = Number.MAX_VALUE;
    var result = null;
    palette.forEach(function (palComps, id) {
        var dist = RgbColor.getDistance(comps, palComps);
        if (dist < minDist) {
            result = id;
            minDist = dist;
        }
    });
    return result;
}
function quantize(rgbaHexes) {
    // Quantize
    var opaqueHexes = extractOpaqueColors(rgbaHexes);
    var colMap = Quantize(opaqueHexes.map(RgbColor.componentsFromHex), 16);
    // Convert pixels into color indexes
    var palette = colMap.palette();
    var colIndexes = rgbaHexes.map(function (rgbaHex) {
        var rgbaComps = RgbaColor.componentsFromHex(rgbaHex);
        var rgbComps = [
            rgbaComps[0],
            rgbaComps[1],
            rgbaComps[2],
        ];
        if (!isOpaque(rgbaComps[3])) {
            return exports.COLOR_INDEX_TRANSPARENT;
        }
        return getNearestColorIndex(colMap, palette, rgbComps) || 0;
    });
    return {
        image: colIndexes,
        palette: palette.map(RgbColor.hexFromComponents),
    };
}
exports.quantize = quantize;
